package com.tykj.dev.device.user.subject.dao;

import com.tykj.dev.device.user.subject.entity.User;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface UserDao extends JpaRepository<User, Integer>, JpaSpecificationExecutor<User> {
 User findByUsername(String userName);

 List<User> findAllByUnitsId(Integer unitsId);

 List<User> findAllByUnitsIdIn(List<Integer> ids);

 List<User> findAllByUserIdIn(List<Integer> userIds);
}
